"""
This file is part of Latynkatar.

Latynkatar is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License v3 (LGPLv3) as published by the 
Free Software Foundation, either version 3 of the License, or (at your option) 
any later version.

Latynkatar is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Lesser General Public License v3 (LGPLv3) for 
more details. In file LICENSE which should came with a package, or look at it
in the repo, see <https://github.com/measles/latynkatar/blob/main/LICENSE>.

You should have received a copy of the GNU Lesser General Public License v3 
(LGPLv3) along with Latynkatar. If not, see <https://www.gnu.org/licenses/>. 

:copyright: (c) 2023, 2024 by Andrej Zacharevicz.
"""

HALOSNYJA = ("а", "е", "ё", "і", "у", "ы", "э", "ю", "я")
MOHUC_PAZNACZACCA_JAK_MIAKKIJA = {
    "н": ("n", "ń"),
    "с": ("s", "ś"),
    "ц": ("c", "ć"),
    "з": ("z", "ź"),
}
PRAVILY_KANVERTACYJ = {
    "а": "a",
    "э": "e",
    "ы": "y",
    "у": "u",
    "ў": "ǔ",
    "б": "b",
    "в": "v",
    "г": "h",
    "ґ": "g",
    "д": "d",
    "ж": "ž",
    "й": "j",
    "к": "k",
    "м": "m",
    "о": "o",
    "п": "p",
    "р": "r",
    "т": "t",
    "ф": "f",
    "ч": "č",
    "ш": "š",
}
PRAVILY_KANVERTACYJ_Z_J = {
    "е": "e",
    "ё": "o",
    "і": "",
    "ю": "u",
    "я": "a",
}



# Try to import from module, else import from the source code
try:
    from latynkatar import Cyr2Lat
except ModuleNotFoundError:
    from src.latynkatar import Cyr2Lat


def test_l():
    assert Cyr2Lat.convert("ЛаЭлЯЛуЛіЛюЛЁлЕлЬ") == "ŁaElAŁuLiLuLOlEl"



"""
This file is part of Latynkatar.

Latynkatar is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License v3 (LGPLv3) as published by the 
Free Software Foundation, either version 3 of the License, or (at your option) 
any later version.

Latynkatar is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Lesser General Public License v3 (LGPLv3) for 
more details. In file LICENSE which should came with a package, or look at it
in the repo, see <https://github.com/measles/latynkatar/blob/main/LICENSE>.

You should have received a copy of the GNU Lesser General Public License v3 
(LGPLv3) along with Latynkatar. If not, see <https://www.gnu.org/licenses/>. 

:copyright: (c) 2023, 2024 by Andrej Zacharevicz.
"""

from .latynkatar import Cyr2Lat



# Try to import from module, else import from the source code
try:
    from latynkatar import Cyr2Lat
except ModuleNotFoundError:
    from src.latynkatar import Cyr2Lat


# Узор узяты з часопіса PAMYŁKA:
# https://github.com/PAMYLKA-ZIN/pamylka-number-3/tree/main/PAMYLKA_ZIN_3_FOR_SHARING
PRYKŁAD_PAMYŁKA = """Вітаем цябе, чытачу!
Гэта трэці нумар PAMYŁKA ZIN!
Мы вельмі цешымся, што да каманды стваральнікаў працягваюць далучацца
новыя навукоўцы і мастакі! І мы будзем радыя кожнаму новаму ўдзельніку!
Сябры, мы рэдакцыяй надумалі запачаткаваць прэмію – «Бізон Гіґс». Таму гэты
нумар мы прысвячаем усім беларускім навукоўцам і хочам анансаваць прэмію,
якая будзе ўвасабляць сабой Беларусь і навуку разам! Гэта ўзнагарода для
беларускіх навукоўцаў і даследнікаў дакладных і прыродазнаўчых навук ад
навукова-папулярнага часопісу Pamyłka Zin.
Больш дэталяў апавядае першы артыкул нумару.
"""
UZOR_PAMYŁKA = """Vitajem ciabie, čytaču!
Heta treci numar PAMYŁKA ZIN!
My vielmi ciešymsia, što da kamandy stvaralnikaǔ praciahvajuć dałučacca
novyja navukoǔcy i mastaki! I my budziem radyja kožnamu novamu ǔdzielniku!
Siabry, my redakcyjaj nadumali zapačatkavać premiju – «Bizon Higs». Tamu hety
numar my prysviačajem usim biełaruskim navukoǔcam i chočam anansavać premiju,
jakaja budzie ǔvasablać saboj Biełaruś i navuku razam! Heta ǔznaharoda dla
biełaruskich navukoǔcaǔ i daslednikaǔ dakładnych i pryrodaznaǔčych navuk ad
navukova-papularnaha časopisu Pamyłka Zin.
Bolš detalaǔ apaviadaje pieršy artykuł numaru.
"""

PRYKŁAD_BAHDANOVIČ = """Маладыя гады,
Маладыя жаданні!
Ні жуды, ні нуды,
Толькі шчасьце каханьня!
 
Помніш толькі красу,
Мілы тварык дзявочы,
Залатую касу,
Сіняватыя вочы!

Цёмны сад-вінаград,
Цьвет бяленькі вішнёвы, —
І агністы пагляд,
І гарачыя словы!
 
Будзь жа, век малады,
Поўны сьветлымі днямі!
Пралятайце, гады,
Залатымі агнямі!
"""
UZOR_BAHDANOVIČ = """Maładyja hady,
Maładyja žadanni!
Ni žudy, ni nudy,
Tolki ščaście kachańnia!
 
Pomniš tolki krasu,
Miły tvaryk dziavočy,
Załatuju kasu,
Siniavatyja vočy!

Ciomny sad-vinahrad,
Ćviet bialeńki višniovy, —
I ahnisty pahlad,
I haračyja słovy!
 
Budź ža, viek małady,
Poǔny śvietłymi dniami!
Pralatajcie, hady,
Załatymi ahniami!
"""


def test_z_pamylki():
    assert Cyr2Lat.convert(PRYKŁAD_PAMYŁKA) == UZOR_PAMYŁKA


def test_bahdanovicz():
    assert Cyr2Lat.convert(PRYKŁAD_BAHDANOVIČ) == UZOR_BAHDANOVIČ



"""
This file is part of Latynkatar.

Latynkatar is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License v3 (LGPLv3) as published by the 
Free Software Foundation, either version 3 of the License, or (at your option) 
any later version.

Latynkatar is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Lesser General Public License v3 (LGPLv3) for 
more details. In file LICENSE which should came with a package, or look at it
in the repo, see <https://github.com/measles/latynkatar/blob/main/LICENSE>.

You should have received a copy of the GNU Lesser General Public License v3 
(LGPLv3) along with Latynkatar. If not, see <https://www.gnu.org/licenses/>. 

:copyright: (c) 2023, 2024 by Andrej Zacharevicz.
"""

from .variables import (
    PRAVILY_KANVERTACYJ,
    HALOSNYJA,
    PRAVILY_KANVERTACYJ_Z_J,
    MOHUC_PAZNACZACCA_JAK_MIAKKIJA,
)


class Cyr2Lat:
    @classmethod
    def convert(cls, text: str) -> str:
        converted_text = ""
        text_length = len(text)
        for index in range(text_length):
            if index > 0:
                previous_letter = text[index - 1]
            else:
                previous_letter = None

            current_letter = text[index]
            if index < text_length - 1:
                next_letter = text[index + 1]
            else:
                next_letter = None

            if current_letter.lower() in PRAVILY_KANVERTACYJ:
                converted_letter = PRAVILY_KANVERTACYJ[current_letter.lower()]
                if current_letter.isupper():
                    converted_letter = converted_letter.upper()

                converted_text += converted_letter
            elif current_letter.lower() == "л":
                if next_letter and next_letter.lower() in ("ь",) + tuple(
                    PRAVILY_KANVERTACYJ_Z_J.keys()
                ):
                    converted_text += "l" if current_letter.islower() else "L"
                else:
                    converted_text += "ł" if current_letter.islower() else "Ł"
            # могуць змякчацца асобна ад галосных літар пасля іх (мяккі знак)
            elif current_letter.lower() in MOHUC_PAZNACZACCA_JAK_MIAKKIJA:
                hard, soft = MOHUC_PAZNACZACCA_JAK_MIAKKIJA[current_letter.lower()]
                if next_letter and next_letter.lower() == "ь":
                    converted_text += soft if current_letter.islower() else soft.upper()
                else:
                    converted_text += hard if current_letter.islower() else hard.upper()
            elif current_letter.lower() == "х":
                converted_text += "ch" if current_letter.islower() else "Ch"
            elif current_letter.lower() == "ь" or current_letter.lower() == "'":
                pass
            # Перадаюцца праз i/j
            elif current_letter.lower() in PRAVILY_KANVERTACYJ_Z_J:
                if (
                    not previous_letter
                    or previous_letter.lower() in HALOSNYJA
                    or not previous_letter.isalpha()
                    or previous_letter == "'"
                ) and current_letter.lower() != "і":
                    base = "j" if current_letter.islower() else "J"
                else:
                    base = "i" if current_letter.islower() else "I"

                second_letter = PRAVILY_KANVERTACYJ_Z_J[current_letter.lower()]
                if (
                    current_letter.lower() != "і"
                    and previous_letter
                    and previous_letter.lower() == "л"
                ):
                    converted_letter = (
                        second_letter
                        if current_letter.islower()
                        else second_letter.upper()
                    )
                else:
                    converted_letter = base + second_letter

                converted_text += converted_letter
            else:
                converted_text += current_letter

        return converted_text
